@@ -1,6 +1,9 @@
 #ifndef __GCMD_H
 #define __GCMD_H
 
+#include "panel.h"
+#include "dir.h" /* sortfn */
+
 void gnome_listing_cmd        	  (GtkWidget *widget, WPanel *panel);
 void gnome_compare_panels     	  (void);
 void gnome_open_terminal      	  (void);
@@ -14,6 +17,17 @@ void gnome_brief_view_cmd         (GtkWidget *widget, WPanel *panel);
 void gnome_detailed_view_cmd      (GtkWidget *widget, WPanel *panel);
 void gnome_custom_view_cmd        (GtkWidget *widget, WPanel *panel);
 
+typedef enum { 
+	SORT_NAME,
+	SORT_EXTENSION,
+	SORT_ACCESS,
+	SORT_MODIFY,
+	SORT_CHANGE,
+	SORT_SIZE
+} SortType; /* Used for arrange icons */
+
+sortfn *sort_get_func_from_type (SortType type);
+
 void gnome_sort_cmd               (GtkWidget *widget, WPanel *panel);
 void gnome_select_all_cmd         (GtkWidget *widget, WPanel *panel);
 void gnome_start_search		  (GtkWidget *widget, WPanel *panel);
@@ -23,7 +37,7 @@ void gnome_open_files             (GtkWidget *widget, WPanel *panel);
 void gnome_run_new                (GtkWidget *widget, GnomeDesktopEntry *gde);
 void gnome_mkdir_cmd              (GtkWidget *widget, WPanel *panel);
 void gnome_new_launcher           (GtkWidget *widget, WPanel *panel);
-void gnome_reverse_selection_cmd_panel (WPanel *panel);
+void gnome_reverse_selection_cmd_panel (GtkWidget *widget, WPanel *panel);
 void gnome_select                 (GtkWidget *widget, WPanel *panel);
 void set_cursor_normal		  (WPanel *panel);
 void set_cursor_busy		  (WPanel *panel);
