@@ -42,14 +42,19 @@ struct layout_slot {
 /* Configuration options for the desktop */
 
 int desktop_use_shaped_icons = TRUE;
+int desktop_use_shaped_text = FALSE;
 int desktop_auto_placement = FALSE;
 int desktop_snap_icons = FALSE;
+int desktop_arr_r2l = FALSE;
+int desktop_arr_b2t = FALSE;
+int desktop_arr_rows = FALSE;
 
 /* The computed name of the user's desktop directory */
 char *desktop_directory;
 
-/* Layout information:  number of rows/columns for the layout slots, and the array of slots.  Each
- * slot is an integer that specifies the number of icons that belong to that slot.
+/* Layout information:  number of rows/columns for the layout slots, and the
+ * array of slots.  Each slot is an integer that specifies the number of icons
+ * that belong to that slot.
  */
 static int layout_screen_width;
 static int layout_screen_height;
@@ -129,6 +134,8 @@ static int click_dragging;
 static DesktopIconInfo *desktop_icon_info_new (char *filename, char *url, char *caption,
 					       int xpos, int ypos);
 
+static GHashTable *icon_hash;
+
 
 /* Convenience function to figure out the slot corresponding to an (x, y) position */
 static void
@@ -143,30 +150,82 @@ get_slot_from_pos (int x, int y, int *u, int *v)
 	*v = CLAMP (vv, 0, layout_rows - 1);
 }
 
+/* Swaps two integer variables */
+static void
+swap (int *a, int *b)
+{
+	int tmp;
+
+	tmp = *a;
+	*a = *b;
+	*b = tmp;
+}
+
 /* Looks for a free slot in the layout_slots array and returns the coordinates
  * that coorespond to it.  "Free" means it either has zero icons in it, or it
  * has the minimum number of icons of all the slots.  Returns the number of
  * icons in the sought spot (ideally 0).
  */
 static int
-auto_pos (int start_slot, int end_slot, int *slot)
+auto_pos (int sx, int ex, int sy, int ey, int *slot)
 {
 	int min, min_slot;
-	int i;
+	int x, y;
 	int val;
+	int xinc, yinc;
+	int r, b;
 
-	min = layout_slots[start_slot].num_icons;
-	min_slot = start_slot;
+	min = l_slots (sx, sy).num_icons;
+	min_slot = sx * layout_rows + sy;
+	xinc = yinc = 1;
 
-	for (i = start_slot; i < end_slot; i++) {
-		val = layout_slots[i].num_icons;
+	if (desktop_arr_rows) {
+		swap (&ex, &ey);
+		swap (&sx, &sy);
+	}
 
-		if (val < min || val == 0) {
-			min = val;
-			min_slot = i;
-			if (val == 0)
-				break;
+#if 0
+	if (desktop_arr_r2l)
+		swap (&sx, &ex);
+
+	if (desktop_arr_b2t)
+		swap (&sy, &ey);
+#endif
+	if (desktop_arr_r2l)
+		xinc = -1;
+
+	if (desktop_arr_b2t)
+		yinc = -1;
+
+	if (desktop_arr_rows)
+		swap (&xinc, &yinc);
+
+	r = desktop_arr_r2l;
+	b = desktop_arr_b2t;
+
+	if (desktop_arr_rows)
+		swap (&r, &b);
+
+	for (x = sx; (r ? (x >= ex) : (x <= ex)); x += xinc) {
+		for (y = sy; (b ? (y >= ey) : (y <= ey)); y += yinc) {
+			if (desktop_arr_rows)
+				val = l_slots (y, x).num_icons;
+			else
+				val = l_slots (x, y).num_icons;
+
+			if (val < min || val == 0) {
+				min = val;
+				if (desktop_arr_rows)
+					min_slot = (y * layout_rows + x);
+				else
+					min_slot = (x * layout_rows + y);
+				if (val == 0)
+					break;
+			}
 		}
+
+		if (val == 0)
+			break;
 	}
 
 	*slot = min_slot;
@@ -177,29 +236,63 @@ auto_pos (int start_slot, int end_slot, int *slot)
 static void
 get_icon_auto_pos (int *x, int *y)
 {
-	int start, end;
-	int u, v;
-	int val1, val2;
-	int slot1, slot2;
+	int val1;
+	int slot1;
 	int slot;
+	int sx, sy, ex, ey;
 
-	get_slot_from_pos (*x, *y, &u, &v);
-	start = u * layout_rows + v;
-	end = layout_cols * layout_rows;
+#if 0
+	get_slot_from_pos (*x, *y, &sx, &sy);
+#endif
+	/* FIXME funky stuff going on with the efficient positioning thingy */
+	if (desktop_arr_r2l)
+		sx = layout_cols - 1;
+	else
+		sx = 0;
+
+	if (desktop_arr_b2t)
+		sy = layout_rows - 1;
+	else
+		sy = 0;
+
+	if (desktop_arr_r2l)
+		ex = 0;
+	else
+		ex = layout_cols - 1;
+
+	if (desktop_arr_b2t)
+		ey = 0;
+	else
+		ey = layout_rows - 1;
 
 	/* Look forwards until the end of the grid.  If we could not find an
 	 * empty spot, find the second best.
 	 */
 
-	val1 = auto_pos (start, end, &slot1);
+	val1 = auto_pos (sx, ex, sy, ey, &slot1);
 
+	slot = slot1;
+
+#if 0
+	/* to be used at a later date */
 	if (val1 == 0)
 		slot = slot1;
 	else {
-		val2 = auto_pos (0, start, &slot2);
+		if (desktop_arr_r2l)
+			sx = layout_cols - 1;
+		else
+			sx = 0;
+
+		if (desktop_arr_b2t)
+			sy = layout_rows - 1;
+		else
+			sy = 0;
+
+		val2 = auto_pos (sx, ex, sy, ey, &slot2);
 		if (val2 < val1)
 			slot = slot2;
 	}
+#endif
 
 	*x = (slot / layout_rows) * DESKTOP_SNAP_X;
 	*y = (slot % layout_rows) * DESKTOP_SNAP_Y;
@@ -309,6 +402,8 @@ desktop_icon_info_destroy (DesktopIconInfo *dii)
 	gtk_widget_destroy (dii->dicon);
 	remove_from_slot (dii);
 
+	g_hash_table_remove (icon_hash, dii->filename);
+
 	g_free (dii->url);
 	g_free (dii->filename);
 	g_free (dii);
@@ -492,10 +587,13 @@ desktop_reload_icons (int user_pos, int xpos, int ypos)
 
 			fau = g_new0 (file_and_url_t, 1);
 			fau->filename = g_strdup (dirent->d_name);
+
 			if (desktop_url)
-				fau->url      = g_strdup (desktop_url);
+				fau->url = g_strdup (desktop_url);
+
 			if (caption)
 				fau->caption  = g_strdup (caption);
+
 			need_position_list = g_slist_prepend (need_position_list, fau);
 		}
 
@@ -610,25 +708,39 @@ desktop_reload_icons (int user_pos, int xpos, int ypos)
 	x_flush_events ();
 }
 
+static WPanel *create_panel_from_desktop(); /* Fwd decl */
+static void free_panel_from_desktop(WPanel *panel);
+
 /* Perform automatic arrangement of the desktop icons */
-void
-desktop_arrange_icons (void)
+static void
+desktop_arrange_icons (SortType type)
 {
-	GList *icons, *l;
+	WPanel *panel;
+	sortfn *sfn;
+	DesktopIconInfo *dii;
+	int i;
+	dir_list dir;
 	int xpos, ypos;
 
-	icons = g_list_reverse (get_all_icons ());
+	panel = create_panel_from_desktop ();
+	sfn = sort_get_func_from_type (type);
+	g_assert (sfn != NULL);
 
-	for (l = icons; l; l = l->next)
-		remove_from_slot (l->data);
+	do_sort (&panel->dir, sfn, panel->count - 1, FALSE, FALSE);
+	dir = panel->dir;
+	g_assert (dir.list != NULL);
 
-	for (l = icons; l; l = l->next) {
+	for (i = 0; i < dir.size; i++)
+		remove_from_slot (desktop_icon_info_get_by_filename (dir.list[i].fname));
+
+	for (i = 0; i < dir.size; i++) {
+		dii = desktop_icon_info_get_by_filename (dir.list[i].fname);
 		xpos = ypos = 0;
 		get_icon_auto_pos (&xpos, &ypos);
-		desktop_icon_info_place (l->data, xpos, ypos);
+		desktop_icon_info_place (dii, xpos, ypos);
 	}
 
-	g_list_free (icons);
+	free_panel_from_desktop (panel);
 }
 
 /* Unselects all the desktop icons except the one in exclude */
@@ -804,8 +916,16 @@ set_icon_wmclass (DesktopIconInfo *dii)
 	XFree (h);
 }
 
-/*
- * Callback used when an icon's text changes.  We must validate the
+/* Removes the Gtk and Gdk grabs that are present when editing a desktop icon */
+static void
+remove_editing_grab (DesktopIconInfo *dii)
+{
+	gtk_grab_remove (dii->dicon);
+	gdk_pointer_ungrab (GDK_CURRENT_TIME);
+	gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+}
+
+/* Callback used when an icon's text changes.  We must validate the
  * rename and return the appropriate value.  The desktop icon info
  * structure is passed in the user data.
  */
@@ -821,6 +941,7 @@ text_changed (GnomeIconTextItem *iti, gpointer data)
 	int retval;
 
 	dii = data;
+	remove_editing_grab (dii);
 
 	source = g_concat_dir_and_file (desktop_directory, dii->filename);
 	new_name = gnome_icon_text_item_get_text (iti);
@@ -830,12 +951,29 @@ text_changed (GnomeIconTextItem *iti, gpointer data)
 
 		dest = g_concat_dir_and_file (desktop_directory, new_name);
 
-		if (mc_rename (source, dest) == 0) {
-			gnome_metadata_delete (dest);
-			gnome_metadata_rename (source, dest);
+		if (rename_file_with_context (source, dest) == FILE_CONT) {
+			GList *icons;
+			GList *l;
 
+			/* See if there was an icon for the new name.  If so,
+			 * destroy it first; desktop_reload_icons() will not be
+			 * happy if two icons have the same filename.
+			 */
+
+			icons = get_all_icons ();
+			l = icon_exists_in_list (icons, new_name);
+			if (l)
+				desktop_icon_info_destroy (l->data);
+
+			g_list_free (icons);
+
+			/* Set the new name */
+
+			g_hash_table_remove (icon_hash, dii->filename);
 			g_free (dii->filename);
+
 			dii->filename = g_strdup (new_name);
+			g_hash_table_insert (icon_hash, dii->filename, dii);
 			set_icon_wmclass (dii);
 
 			desktop_reload_icons (FALSE, 0, 0);
@@ -928,10 +1066,7 @@ editing_stopped (GnomeIconTextItem *iti, gpointer data)
 	DesktopIconInfo *dii;
 
 	dii = data;
-
-	gtk_grab_remove (dii->dicon);
-	gdk_pointer_ungrab (GDK_CURRENT_TIME);
-	gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+	remove_editing_grab (dii);
 
 	/* Re-enable drags from this icon */
 
@@ -1401,21 +1536,9 @@ desktop_icon_set_busy (DesktopIconInfo *dii, int busy)
 DesktopIconInfo *
 desktop_icon_info_get_by_filename (char *filename)
 {
-	int i;
-	GList *l;
-	DesktopIconInfo *dii;
-
 	g_return_val_if_fail (filename != NULL, NULL);
 
-	for (i = 0; i < layout_cols * layout_rows; i++)
-		for (l = layout_slots[i].icons; l; l = l->next) {
-			dii = l->data;
-
-			if (strcmp (dii->filename, filename) == 0)
-				return dii;
-		}
-
-	return NULL;
+	return g_hash_table_lookup (icon_hash, filename);
 }
 
 /* Used to execute the popup menu for desktop icons */
@@ -1916,12 +2039,12 @@ icon_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gin
 
 		full_name = g_concat_dir_and_file (desktop_directory, dii->filename);
 		fe = file_entry_from_file (full_name);
-		if (!fe){
+		if (!fe) {
 			g_free (full_name);
 			return;
 		}
 
-		if (gdnd_perform_drop (context, data, desktop_directory, fe))
+		if (gdnd_perform_drop (context, data, full_name, fe))
 			desktop_reload_icons (FALSE, 0, 0);
 
 		file_entry_free (fe);
@@ -2038,8 +2161,11 @@ desktop_icon_info_new (char *filename, char *url, char *caption, int xpos, int y
 	setup_icon_dnd_dest (dii);
 
 	/* Place the icon and append it to the list */
-
 	desktop_icon_info_place (dii, xpos, ypos);
+
+	/* Put into icon hash table */
+	g_hash_table_insert (icon_hash, dii->filename, dii);
+
 	return dii;
 }
 
@@ -2208,7 +2334,7 @@ setup_motif_dnd_proxy (guint32 xid, GdkWindow *proxy_window)
 	info.protocol_style = 5; /* XmDRAG_DYNAMIC */
 	info.proxy_window = proxy_xid;
 	info.num_drop_sites = 0;
-	info.total_size = sizeof(info);
+	info.total_size = sizeof (info);
 
 	XChangeProperty (gdk_display, xid,
 			 receiver_info_atom,
@@ -2371,11 +2497,46 @@ find_click_proxy_window (void)
 	return proxy_gdk_window;
 }
 
-/* Callback for arranging the icons on the desktop */
+/* Callback for arranging icons by name */
 static void
-handle_arrange_icons (GtkWidget *widget, gpointer data)
+handle_arrange_icons_name (GtkWidget *widget, gpointer data)
 {
-	desktop_arrange_icons ();
+	desktop_arrange_icons (SORT_NAME);
+}
+
+/* Callback for arranging icons by file type */
+static void
+handle_arrange_icons_type (GtkWidget *widget, gpointer data)
+{
+	desktop_arrange_icons (SORT_EXTENSION);
+}
+
+/* Callback for arranging icons by file size */
+static void
+handle_arrange_icons_size (GtkWidget *widget, gpointer data)
+{
+	desktop_arrange_icons (SORT_SIZE);
+}
+
+/* Callback for arranging icons by access time */
+static void
+handle_arrange_icons_access (GtkWidget *widget, gpointer data)
+{
+	desktop_arrange_icons (SORT_ACCESS);
+}
+
+/* Callback for arranging icons by modification time */
+static void
+handle_arrange_icons_mod (GtkWidget *widget, gpointer data)
+{
+	desktop_arrange_icons (SORT_MODIFY);
+}
+
+/* Callback for arranging icons by change time */
+static void
+handle_arrange_icons_change (GtkWidget *widget, gpointer data)
+{
+	desktop_arrange_icons (SORT_CHANGE);
 }
 
 /* Callback for creating a new panel window */
@@ -2385,13 +2546,12 @@ handle_new_window (GtkWidget *widget, gpointer data)
 	new_panel_at (gnome_user_home_dir);
 }
 
+/* Rescans the mountable devices in the desktop and re-creates their icons */
 void
 desktop_rescan_devices (void)
 {
 	desktop_cleanup_devices ();
-
 	gdesktop_init ();
-
 	desktop_reload_icons (FALSE, 0, 0);
 }
 
@@ -2447,46 +2607,79 @@ new_terminal (GtkWidget *widget, gpointer data)
 }
 
 static GnomeUIInfo gnome_panel_new_menu [] = {
-	GNOMEUIINFO_ITEM_NONE(N_("_Terminal"), N_("Launch a new terminal in the current directory"), new_terminal),
+	GNOMEUIINFO_ITEM_NONE (N_("_Terminal"), N_("Launch a new terminal in the current directory"), new_terminal),
 	/* If this ever changes, make sure you update create_new_menu accordingly. */
-	GNOMEUIINFO_ITEM_NONE( N_("_Directory..."), N_("Creates a new directory"), gnome_mkdir_cmd ),
-	GNOMEUIINFO_ITEM_NONE( N_("URL L_ink..."),  N_("Creates a new URL link"), gnome_new_link ),
-	GNOMEUIINFO_ITEM_NONE( N_("_Launcher..."), N_("Creates a new launcher"), gnome_new_launcher ),
+	GNOMEUIINFO_ITEM_NONE (N_("_Directory..."), N_("Creates a new directory"), gnome_mkdir_cmd),
+	GNOMEUIINFO_ITEM_NONE (N_("URL L_ink..."),  N_("Creates a new URL link"), gnome_new_link),
+	GNOMEUIINFO_ITEM_NONE (N_("_Launcher..."), N_("Creates a new launcher"), gnome_new_launcher),
+	GNOMEUIINFO_END
+};
+
+/* Menu items for arranging the desktop icons */
+GnomeUIInfo desktop_arrange_icons_items[] = {
+	GNOMEUIINFO_ITEM_NONE (N_("By _Name"), NULL, handle_arrange_icons_name),
+	GNOMEUIINFO_ITEM_NONE (N_("By File _Type"), NULL, handle_arrange_icons_type),
+	GNOMEUIINFO_ITEM_NONE (N_("By _Size"), NULL, handle_arrange_icons_size),
+	GNOMEUIINFO_ITEM_NONE (N_("By Time Last _Accessed"), NULL, handle_arrange_icons_access),
+	GNOMEUIINFO_ITEM_NONE (N_("By Time Last _Modified"), NULL, handle_arrange_icons_mod),
+	GNOMEUIINFO_ITEM_NONE (N_("By Time Last _Changed"), NULL, handle_arrange_icons_change),
 	GNOMEUIINFO_END
 };
 
 /* The popup menu for the desktop */
 GnomeUIInfo desktop_popup_items[] = {
-	GNOMEUIINFO_MENU_NEW_SUBTREE(gnome_panel_new_menu),
+	GNOMEUIINFO_MENU_NEW_SUBTREE (gnome_panel_new_menu),
 	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_ITEM_NONE (N_("Arrange Icons"), NULL, handle_arrange_icons),
-	GNOMEUIINFO_ITEM_NONE (N_("Create New Window"), NULL, handle_new_window),
+	GNOMEUIINFO_SUBTREE (N_("_Arrange Icons"), desktop_arrange_icons_items),
+	GNOMEUIINFO_ITEM_NONE (N_("Create _New Window"), NULL, handle_new_window),
 	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_ITEM_NONE (N_("Recreate Desktop Shortcuts"), NULL, handle_rescan_devices),
-	GNOMEUIINFO_ITEM_NONE (N_("Rescan Desktop"), NULL, handle_rescan_desktop),
-	GNOMEUIINFO_ITEM_NONE (N_("Configure Background Image"), NULL, set_background_image),
+	GNOMEUIINFO_ITEM_NONE (N_("Recreate Desktop _Shortcuts"), NULL, handle_rescan_devices),
+	GNOMEUIINFO_ITEM_NONE (N_("Rescan _Desktop"), NULL, handle_rescan_desktop),
+	GNOMEUIINFO_ITEM_NONE (N_("Configure _Background Image"), NULL, set_background_image),
 	GNOMEUIINFO_END
 };
 
+static int
+strip_tearoff_menu_item (GnomeUIInfo *infos)
+{
+	GtkWidget *shell;
+	GList *child_list;
+	int n;
+
+	g_assert (infos != NULL);
+
+	shell = infos[0].widget->parent;
+	child_list = gtk_container_children (GTK_CONTAINER (shell));
+	n = g_list_length (child_list);
+
+	if (child_list && GTK_IS_TEAROFF_MENU_ITEM (child_list->data)) {
+		n--;
+		gtk_widget_destroy (GTK_WIDGET (child_list->data));
+#if 0
+		gtk_widget_hide (GTK_WIDGET (child_list->data));
+#endif
+	}
+
+	return n;
+}
+
 /* Executes the popup menu for the desktop */
 static void
 desktop_popup (GdkEventButton *event)
 {
 	GtkWidget *shell;
 	GtkWidget *popup;
-	GList *child_list;
 	gchar *file, *file2;
 	WPanel *panel;
 	gint i;
 
+	/* Create the menu and then strip the tearoff menu items, sigh... */
+
 	popup = gnome_popup_menu_new (desktop_popup_items);
-	/* First thing we want to do is strip off the STUPID tear off menu... S-: */
+
+	strip_tearoff_menu_item (desktop_arrange_icons_items);
+	i = strip_tearoff_menu_item (gnome_panel_new_menu);
 	shell = gnome_panel_new_menu[0].widget->parent;
-	child_list = gtk_container_children (GTK_CONTAINER (shell));
-	if (GTK_IS_TEAROFF_MENU_ITEM (child_list->data))
-		gtk_widget_hide (GTK_WIDGET (child_list->data));
-	i = g_list_length (child_list);
-	g_list_free (child_list);
 	file = gnome_unconditional_datadir_file ("mc/templates");
 	i = create_new_menu_from (file, shell, i);
 	file2 = gnome_datadir_file ("mc/templates");
@@ -2648,7 +2841,6 @@ update_drag_selection (int x, int y)
 				desktop_icon_select (DESKTOP_ICON (dii->dicon), dii->tmp_selected);
 				dii->selected = dii->tmp_selected;
 			}
-
 		}
 }
 
@@ -2658,6 +2850,29 @@ click_proxy_button_press (GtkWidget *widget, GdkEventButton *event, gpointer dat
 {
 	GdkCursor *cursor;
 
+	/* maybe the user wants to click on the icon text */
+	if (event->button == 1 && desktop_use_shaped_text) {
+		int x = event->x;
+		int y = event->y;
+		GList *l, *icons = get_all_icons ();
+		DesktopIconInfo *clicked = NULL;
+		for (l = icons; l; l = l->next) {
+			DesktopIconInfo *dii = l->data;
+			DesktopIcon *di = DESKTOP_ICON (dii->dicon);
+			int x1 = dii->x + di->text_x;
+			int y1 = dii->y + di->text_y;
+			int x2 = x1 + di->text_w;
+			int y2 = y1 + di->text_h;
+			if (x>=x1 && y>=y1 && x<=x2 && y<=y2)
+				clicked = dii;
+		}
+		g_list_free (icons);
+		if (clicked) {
+			select_icon (clicked, event->state);
+			return FALSE;
+		}
+	}
+	
 	if (event->button == 1) {
 		click_start_x = event->x;
 		click_start_y = event->y;
@@ -2922,6 +3137,7 @@ setup_desktop_clicks (void)
 	gdk_bitmap_unref (stipple);
 }
 
+
 /**
  * desktop_init
  *
@@ -2931,6 +3147,8 @@ setup_desktop_clicks (void)
 void
 desktop_init (void)
 {
+	icon_hash = g_hash_table_new (g_str_hash, g_str_equal);
+
 	gdnd_init ();
 	gicon_init ();
 	create_layout_info ();
@@ -2976,6 +3194,8 @@ desktop_destroy (void)
 
 	gtk_widget_destroy (proxy_invisible);
 	XDeleteProperty (GDK_DISPLAY (), GDK_ROOT_WINDOW (), gdk_atom_intern ("XdndProxy", FALSE));
+
+	g_hash_table_destroy (icon_hash);
 }
 
 void
