@@ -395,13 +395,17 @@ GnomeUIInfo gnome_panel_edit_menu [] = {
 	{ GNOME_APP_UI_ITEM,  N_("Select _All"),        N_("Select all files in the current Panel"), gnome_select_all_cmd,
 	  NULL, NULL, 0, NULL, 'a', GDK_CONTROL_MASK  },
 	GNOMEUIINFO_ITEM_NONE(N_("_Select Files..."),  N_("Select a group of files"), gnome_select),
-	GNOMEUIINFO_ITEM_NONE(N_("_Invert Selection"), N_("Reverses the list of tagged files"), reverse_selection_cmd),
+	GNOMEUIINFO_ITEM_NONE(N_("_Invert Selection"), N_("Reverses the list of tagged files"),
+			      gnome_reverse_selection_cmd_panel),
 	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_ITEM, N_("Search"),		N_("Search for a file in the current Panel"), gnome_start_search,
 	  NULL, NULL, 0, NULL, 's', GDK_CONTROL_MASK  },
 	GNOMEUIINFO_SEPARATOR,
         GNOMEUIINFO_ITEM_NONE(N_("_Rescan Directory"), N_("Rescan the directory contents"), reread_cmd),
-	GNOMEUIINFO_SEPARATOR,
+	GNOMEUIINFO_END
+};
+
+GnomeUIInfo gnome_panel_settings_menu [] = {
 	GNOMEUIINFO_MENU_PREFERENCES_ITEM(gnome_configure_box, NULL),
 	GNOMEUIINFO_END
 };
@@ -442,22 +446,23 @@ GnomeUIInfo gnome_panel_commands_menu [] = {
 
 
 GnomeUIInfo gnome_panel_about_menu [] = {
-	GNOMEUIINFO_HELP ("gmc"),
 	GNOMEUIINFO_MENU_ABOUT_ITEM(gnome_about_cmd, NULL),
+	GNOMEUIINFO_HELP ("gmc"),
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo gnome_panel_desktop_menu [] = {
-	GNOMEUIINFO_ITEM_NONE (N_("Arrange Icons"), NULL, desktop_arrange_icons),
+	GNOMEUIINFO_SUBTREE(N_("_Arrange Icons"), desktop_arrange_icons_items),
 	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_ITEM_NONE (N_("Rescan System Devices"), NULL, desktop_rescan_devices),
-	GNOMEUIINFO_ITEM_NONE (N_("Rescan Desktop"), NULL, do_desktop_rescan_devices),
+	GNOMEUIINFO_ITEM_NONE (N_("Recreate Desktop _Shortcuts"), NULL, desktop_rescan_devices),
+	GNOMEUIINFO_ITEM_NONE (N_("Rescan _Desktop"), NULL, do_desktop_rescan_devices),
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo gnome_panel_menu_with_desktop [] = {
         GNOMEUIINFO_MENU_FILE_TREE(gnome_panel_file_menu),
         GNOMEUIINFO_MENU_EDIT_TREE(gnome_panel_edit_menu),
+	GNOMEUIINFO_SUBTREE(N_("_Settings"),gnome_panel_settings_menu),
         GNOMEUIINFO_SUBTREE(N_("_Layout"),gnome_panel_layout_menu),
         GNOMEUIINFO_SUBTREE(N_("_Commands"),gnome_panel_commands_menu),
 	GNOMEUIINFO_SUBTREE(N_("_Desktop"), gnome_panel_desktop_menu),
@@ -468,6 +473,7 @@ GnomeUIInfo gnome_panel_menu_with_desktop [] = {
 GnomeUIInfo gnome_panel_menu_without_desktop [] = {
         GNOMEUIINFO_MENU_FILE_TREE(gnome_panel_file_menu),
         GNOMEUIINFO_MENU_EDIT_TREE(gnome_panel_edit_menu),
+	GNOMEUIINFO_SUBTREE(N_("_Settings"),gnome_panel_settings_menu),
         GNOMEUIINFO_SUBTREE(N_("_Layout"),gnome_panel_layout_menu),
         GNOMEUIINFO_SUBTREE(N_("_Commands"),gnome_panel_commands_menu),
 	GNOMEUIINFO_SUBTREE(N_("_Help"), gnome_panel_about_menu),
