@@ -27,6 +27,7 @@
 #include "dir.h"
 #include "dialog.h"
 #include "setup.h"
+#include "file.h"
 #include "fileopctx.h"
 #include "gdesktop.h"
 #include "gdnd.h"
@@ -40,6 +41,10 @@
 #include "../vfs/vfs.h"
 #include <gdk/gdkprivate.h>
 
+#ifndef MAX
+#   define MAX(a,b) ((a) > (b) ? a : b)
+#endif
+
 /* Whether to display the tree view on the left */
 int tree_panel_visible = -1;
 
@@ -125,7 +130,7 @@ static void
 panel_cancel_drag_scroll (WPanel *panel)
 {
 	g_return_if_fail (panel != NULL);
-	
+
 	if (panel->timer_id != -1){
 		gtk_timeout_remove (panel->timer_id);
 		panel->timer_id = -1;
@@ -190,11 +195,12 @@ panel_fill_panel_list (WPanel *panel)
 	type_col = -1;
 
 	g_assert (items == cl->columns);
-		  
+
 	texts [items] = NULL;
 	for (i = 0; i < top; i++){
 		file_entry *fe = &panel->dir.list [i];
 		format_e *format = panel->format;
+		int n;
 
 		for (col = 0; format; format = format->next){
 			if (!format->use_in_gui)
@@ -205,12 +211,17 @@ panel_fill_panel_list (WPanel *panel)
 					type_col = col;
 
 			if (!format->string_fn)
-				texts [col] = "";
+				texts[col] = "";
 			else
-				texts [col] = (*format->string_fn)(fe, 10);
+				texts[col] = (* format->string_fn) (fe, 10);
 			col++;
 		}
-		gtk_clist_append (cl, texts);
+
+		n = gtk_clist_append (cl, texts);
+
+		/* Do not let the user select .. */
+		if (strcmp (fe->fname, "..") == 0)
+			gtk_clist_set_selectable (cl, n, FALSE);
 
 		color = file_compute_color (NORMAL, fe);
 		panel_file_list_set_row_colors (cl, i, color);
@@ -220,13 +231,17 @@ panel_fill_panel_list (WPanel *panel)
 
 		if (fe->f.marked)
 			gtk_clist_select_row (cl, i, 0);
-		
+		else
+			gtk_clist_unselect_row (cl, i, 0);
+
 	}
+
+	g_free (texts);
+
 	/* This is needed as the gtk_clist_append changes selected under us :-( */
 	panel->selected = selected;
-	select_item (panel);
+
 	gtk_clist_thaw (GTK_CLIST (cl));
-	 g_free (texts);
 }
 
 /*
@@ -235,40 +250,32 @@ panel_fill_panel_list (WPanel *panel)
 static void
 panel_fill_panel_icons (WPanel *panel)
 {
-	GnomeIconList *icons = ILIST_FROM_SW (panel->icons);
 	const int top       = panel->count;
 	const int selected  = panel->selected;
+	GnomeIconList *icons = ILIST_FROM_SW (panel->icons);
 	int i;
 	GdkImlibImage *image;
 
 	gnome_icon_list_freeze (icons);
 	gnome_icon_list_clear (icons);
-#if 0
-	/*
-	 * HACK_BEGIN:
-	 *    Temporary hack for pre-gnome-libs-1.0.10
-	 *
-	 *    Remove after wildely deployed
-	 */
-	icons->last_selected = 0;
-	icons->last_clicked = 0;
-	/*
-	 * HACK_END:
-	 */
-#endif
+
 	for (i = 0; i < top; i++) {
 		file_entry *fe = &panel->dir.list [i];
 		int p;
-		
+
 		image = gicon_get_icon_for_file (panel->cwd, fe, TRUE);
 		p = gnome_icon_list_append_imlib (icons, image, fe->fname);
+
 		if (fe->f.marked)
 			gnome_icon_list_select_icon (icons, p);
+		else
+			gnome_icon_list_unselect_icon (icons, p);
 	}
-	/* This is needed as the gtk_clist_append changes selected under us :-( */
+
+	/* This is needed as the gtk_gnome_icon_list_append_imlib changes selected under us :-( */
 	panel->selected = selected;
+
 	gnome_icon_list_thaw (icons);
-	select_item (panel);
 }
 
 /*
@@ -420,7 +427,7 @@ panel_file_list_configure_contents (GtkWidget *sw, WPanel *panel, int main_width
 			used_columns += 2;
 		else
 			used_columns += format->field_len;
-		
+
 		items++;
 		if (format->expand)
 			expandables++;
@@ -482,10 +489,6 @@ panel_file_list_configure_contents (GtkWidget *sw, WPanel *panel, int main_width
 	gtk_clist_thaw (clist);
 }
 
-/* Handler for the select_row signal of the clist.  We synchronize the panel's
- * idea of a selection, and handle pending actions that the
- * button_press/button_release handlers did not handle by themselves.
- */
 static void
 panel_file_list_select_row (GtkWidget *file_list, gint row, gint column,
 			    GdkEvent *event, gpointer data)
@@ -513,7 +516,6 @@ panel_file_list_unselect_row (GtkWidget *widget, int row, int columns, GdkEvent
 		panel->selected = 0;
 }
 
-
 /* Figure out the number of visible lines in the panel */
 static void
 panel_file_list_compute_lines (GtkScrolledWindow *sw, WPanel *panel, int height)
@@ -755,6 +757,58 @@ panel_drag_data_delete (GtkWidget *widget, GdkDragContext *context, WPanel *pane
 	/* Things is: The File manager already handles file moving */
 }
 
+/* Performs a drop on a panel.  If idx is -1, then drops on the panel's cwd
+ * itself.
+ */
+static void
+drop_on_panel (WPanel *panel, int idx, GdkDragContext *context, GtkSelectionData *selection_data)
+{
+	char *file;
+	file_entry *fe;
+	int drop_on_dir;
+	int reload;
+
+	g_assert (panel != NULL);
+	g_assert (idx == -1 || idx < panel->count);
+	g_assert (context != NULL);
+	g_assert (selection_data != NULL);
+
+	drop_on_dir = FALSE;
+
+	if (idx == -1)
+		drop_on_dir = TRUE;
+	else {
+		fe = &panel->dir.list[idx];
+		file = g_concat_dir_and_file (panel->cwd, fe->fname);
+
+		if (!((S_ISDIR (fe->buf.st_mode) || fe->f.link_to_dir)
+		      || gdnd_can_drop_on_file (file, fe))) {
+			g_free (file);
+			drop_on_dir = TRUE;
+		}
+	}
+
+	if (drop_on_dir) {
+		file = panel->cwd;
+		fe = file_entry_from_file (file);
+		if (!fe)
+			return; /* eeeek */
+	}
+
+	reload = gdnd_perform_drop (context, selection_data, file, fe);
+
+	if (file != panel->cwd)
+		g_free (file);
+
+	if (drop_on_dir)
+		file_entry_free (fe);
+
+	if (reload) {
+		update_panels (UP_OPTIMIZE, UP_KEEPSEL);
+		repaint_screen ();
+	}
+}
+
 /**
  * panel_icon_list_drag_data_received:
  *
@@ -771,48 +825,10 @@ panel_icon_list_drag_data_received (GtkWidget          *widget,
 				    guint32             time,
 				    WPanel              *panel)
 {
-	GnomeIconList *gil = GNOME_ICON_LIST (widget);
-	file_entry *fe;
-	char *file;
-	int free_file, free_fe;
 	int idx;
-	gboolean reload;
 
-	idx = gnome_icon_list_get_icon_at (gil, x, y);
-	if (idx == -1) {
-		file = panel->cwd;
-		fe = file_entry_from_file (file);
-		if (!fe)
-			return; /* eeeek */
-
-		free_file = FALSE;
-		free_fe = TRUE;
-	} else {
-		fe = &panel->dir.list[idx];
-
-		if (S_ISDIR (fe->buf.st_mode) || fe->f.link_to_dir){
-			file = g_concat_dir_and_file (panel->cwd, panel->dir.list[idx].fname);
-			free_file = TRUE;
-		} else {
-			file = panel->cwd;
-			free_file = FALSE;
-		}
-		
-		free_fe = FALSE;
-	}
-
-	reload = gdnd_perform_drop (context, selection_data, file, fe);
-
-	if (free_file)
-		g_free (file);
-
-	if (free_fe)
-		file_entry_free (fe);
-
-	if (reload) {
-		update_panels (UP_OPTIMIZE, UP_KEEPSEL);
-		repaint_screen ();
-	}
+	idx = gnome_icon_list_get_icon_at (GNOME_ICON_LIST (widget), x, y);
+	drop_on_panel (panel, idx, context, selection_data);
 }
 
 /**
@@ -831,49 +847,21 @@ panel_clist_drag_data_received (GtkWidget          *widget,
 				guint32             time,
 				WPanel              *panel)
 {
-	GtkCList *clist = GTK_CLIST (widget);
-	file_entry *fe;
-	char *file;
-	int free_file, free_fe;
+	GtkCList *clist;
 	int row;
-	int reload;
 
-	if (gtk_clist_get_selection_info (clist, x, y, &row, NULL) == 0) {
-		file = panel->cwd;
-		fe = file_entry_from_file (file);
-		if (!fe)
-			return; /* eeeek */
+	clist = GTK_CLIST (widget);
 
-		free_file = FALSE;
-		free_fe = TRUE;
-	} else {
-		g_assert (row < panel->count);
+	/* Normalize the y coordinate to the clist_window */
 
-		fe = &panel->dir.list[row];
+	y -= (GTK_CONTAINER (clist)->border_width
+	      + clist->column_title_area.y
+	      + clist->column_title_area.height);
 
-		if (S_ISDIR (fe->buf.st_mode) || fe->f.link_to_dir){
-			file = g_concat_dir_and_file (panel->cwd, panel->dir.list[row].fname);
-			free_file = TRUE;
-		} else {
-			file = panel->cwd;
-			free_file = FALSE;
-		}
-		
-		free_fe = FALSE;
-	}
+	if (gtk_clist_get_selection_info (GTK_CLIST (widget), x, y, &row, NULL) == 0)
+		row = -1;
 
-	reload = gdnd_perform_drop (context, selection_data, file, fe);
-
-	if (free_file)
-		g_free (file);
-
-	if (free_fe)
-		file_entry_free (fe);
-
-	if (reload) {
-		update_panels (UP_OPTIMIZE, UP_KEEPSEL);
-		repaint_screen ();
-	}
+	drop_on_panel (panel, row, context, selection_data);
 }
 
 /**
@@ -907,8 +895,10 @@ panel_tree_drag_data_received (GtkWidget          *widget,
 
 	path = gtk_dtree_get_row_path (dtree, node, 0);
 	fe = file_entry_from_file (path);
-	if (!fe)
+	if (!fe) {
+		g_free (path);
 		return; /* eeeek */
+	}
 
 	reload = gdnd_perform_drop (context, selection_data, path, fe);
 
@@ -1069,6 +1059,7 @@ panel_clist_drag_motion (GtkWidget *widget, GdkDragContext *context, gint x, gin
 	GtkWidget *source_widget;
 	gint idx;
 	file_entry *fe;
+	char *full_name;
 
 	panel = data;
 
@@ -1099,14 +1090,19 @@ panel_clist_drag_motion (GtkWidget *widget, GdkDragContext *context, gint x, gin
 	else
 		fe = &panel->dir.list[idx];
 
+	full_name = fe ? g_concat_dir_and_file (panel->cwd, fe->fname) : panel->cwd;
+
 	action = gdnd_validate_action (context,
 				       FALSE,
 				       source_widget != NULL,
 				       source_widget == widget,
-				       panel->cwd,
+				       full_name,
 				       fe,
 				       fe ? fe->f.marked : FALSE);
 
+	if (full_name != panel->cwd)
+		g_free (full_name);
+
 	gdk_drag_status (context, action, time);
 
  out:
@@ -1200,6 +1196,7 @@ panel_icon_list_drag_motion (GtkWidget *widget, GdkDragContext *context, gint x,
 	GtkWidget *source_widget;
 	int idx;
 	file_entry *fe;
+	char *full_name;
 
 	panel = data;
 
@@ -1216,14 +1213,19 @@ panel_icon_list_drag_motion (GtkWidget *widget, GdkDragContext *context, gint x,
 	idx = gnome_icon_list_get_icon_at (GNOME_ICON_LIST (widget), x, y);
 	fe = (idx == -1) ? NULL : &panel->dir.list[idx];
 
+	full_name = fe ? g_concat_dir_and_file (panel->cwd, fe->fname) : panel->cwd;
+
 	action = gdnd_validate_action (context,
 				       FALSE,
 				       source_widget != NULL,
 				       source_widget == widget,
-				       panel->cwd,
+				       full_name,
 				       fe,
 				       fe ? fe->f.marked : FALSE);
 
+	if (full_name != panel->cwd)
+		g_free (full_name);
+
 	gdk_drag_status (context, action, time);
 	return TRUE;
 }
@@ -1376,6 +1378,10 @@ panel_icon_list_select_icon (GtkWidget *widget, int index, GdkEvent *event, WPan
 	display_mini_info (panel);
 	execute_hooks (select_file_hook);
 
+	/* Do not let the user select .. */
+	if (strcmp (panel->dir.list[index].fname, "..") == 0)
+		gnome_icon_list_unselect_icon (GNOME_ICON_LIST (widget), index);
+
 	if (!event)
 		return;
 
@@ -1419,18 +1425,64 @@ panel_icon_list_unselect_icon (GtkWidget *widget, int index, GdkEvent *event, WP
 		panel->selected = 0;
 }
 
+static int
+queue_reread_cmd (gpointer data)
+{
+	reread_cmd ();
+	return FALSE;
+}
+
+/* Renames a file using a file operation context.  Returns FILE_CONT on success. */
+int
+rename_file_with_context (char *source, char *dest)
+{
+	FileOpContext *ctx;
+	struct stat s;
+	long count;
+	double bytes;
+	int retval;
+
+	if (mc_lstat (source, &s) != 0)
+		return FILE_ABORT;
+
+	ctx = file_op_context_new ();
+	file_op_context_create_ui (ctx, OP_MOVE, FALSE);
+
+	count = 1;
+	bytes = s.st_size;
+
+	retval = move_file_file (ctx, source, dest, &count, &bytes);
+	file_op_context_destroy (ctx);
+
+	return retval;
+}
+
 static int
 panel_icon_renamed (GtkWidget *widget, int index, char *dest, WPanel *panel)
 {
 	char *source;
+	char *fullname;
+	int retval;
 
-	source = panel->dir.list [index].fname;
-	if (mc_rename (source, dest) == 0){
+	if (strcmp (dest, panel->dir.list[index].fname) == 0)
+		return TRUE; /* do nothing if the name did not change */
+
+	source = g_concat_dir_and_file (cpanel->cwd, panel->dir.list[index].fname);
+	fullname = g_concat_dir_and_file (cpanel->cwd, dest);
+
+	if (rename_file_with_context (source, dest) == FILE_CONT) {
 		g_free (panel->dir.list [index].fname);
 		panel->dir.list [index].fname = g_strdup (dest);
-		return TRUE;
+
+		gtk_idle_add (queue_reread_cmd, NULL);
+		retval = TRUE;
 	} else
-		return FALSE;
+		retval = FALSE;
+
+	g_free (source);
+	g_free (fullname);
+
+	return retval;
 }
 
 /* Callback for rescanning the cwd */
@@ -1463,6 +1515,21 @@ file_list_popup (GdkEventButton *event, WPanel *panel)
 }
 
 
+/* Returns whether an icon in the icon list is being edited.  FIXME:  This
+ * function uses a fantastically ugly hack to figure this out.  It would be
+ * saner to have a function provided by the icon list widget to do it, but we
+ * can't break forwards compatibility at this point.  It would be even saner to
+ * have a good DnD API for the icon list.
+ */
+static int
+editing_icon_list (GnomeIconList *gil)
+{
+	GnomeCanvasItem *item;
+
+	item = GNOME_CANVAS (gil)->focused_item;
+	return (item && GNOME_IS_ICON_TEXT_ITEM (item) && GNOME_ICON_TEXT_ITEM (item)->editing);
+}
+
 /*
  * Strategy for activaing the drags from the icon-list:
  *
@@ -1478,6 +1545,9 @@ panel_icon_list_button_press (GtkWidget *widget, GdkEventButton *event, WPanel *
 	GnomeIconList *gil = GNOME_ICON_LIST (widget);
 	int icon;
 
+	if (editing_icon_list (gil))
+		return FALSE;
+
 	if (event->type != GDK_BUTTON_PRESS)
 		return FALSE;
 
@@ -1791,7 +1861,10 @@ tree_drag_open_directory (gpointer data)
 	g_assert (node != NULL);
 
 	if (!GTK_CTREE_ROW (node)->expanded) {
+#if 0
+		/* FIXME: Disabled until fully debugged.  Should also be configurable. */
 		dtree->auto_expanded_nodes = g_list_append (dtree->auto_expanded_nodes, node);
+#endif
 		gtk_ctree_expand (GTK_CTREE (panel->tree), node);
 	}
 
@@ -1925,7 +1998,6 @@ panel_tree_drag_motion (GtkWidget *widget, GdkDragContext *context, int x, int y
 	GtkWidget *source_widget;
 	char *row_path;
 	int on_drag_row;
-	char *parent_dir, *p;
 
 	dtree = GTK_DTREE (widget);
 	panel = data;
@@ -1976,13 +2048,6 @@ panel_tree_drag_motion (GtkWidget *widget, GdkDragContext *context, int x, int y
 			panel->drag_tree_row = row;
 		}
 
-		/* Compute the parent directory of the file entry */
-
-		parent_dir = g_strdup (row_path);
-		p = strrchr (parent_dir, PATH_SEP);
-		g_assert (p != NULL);
-		p[1] = 0;
-
 		/* Validate the action */
 
 		gdnd_find_panel_by_drag_context (context, &source_widget);
@@ -1996,16 +2061,15 @@ panel_tree_drag_motion (GtkWidget *widget, GdkDragContext *context, int x, int y
 			on_drag_row = strcmp (row_path, dtree->drag_dir) == 0;
 		} else
 			on_drag_row = FALSE;
-			
+
 		action = gdnd_validate_action (context,
 					       FALSE,
 					       source_widget != NULL,
 					       source_widget == widget,
-					       parent_dir,
+					       row_path,
 					       panel->drag_tree_fe,
 					       on_drag_row);
 
-		g_free (parent_dir);
 		g_free (row_path);
 	} else {
 		panel->drag_tree_row = -1;
@@ -2080,10 +2144,10 @@ static void
 panel_tree_button_press (GtkWidget *widget, GdkEventButton *event, WPanel *panel)
 {
 	GtkWidget *popup;
-	
+
 	if (event->type != GDK_BUTTON_PRESS)
 		return;
-	
+
 	if (event->button != 3)
 		return;
 
@@ -2245,12 +2309,12 @@ static GnomeUIInfo toolbar[] = {
 };
 
 static void
-do_ui_signal_connect (GnomeUIInfo *uiinfo, gchar *signal_name, 
+do_ui_signal_connect (GnomeUIInfo *uiinfo, gchar *signal_name,
 		GnomeUIBuilderData *uibdata)
 {
 	if (uiinfo->moreinfo)
-		gtk_signal_connect (GTK_OBJECT (uiinfo->widget), 
-				    signal_name, uiinfo->moreinfo, uibdata->data ? 
+		gtk_signal_connect (GTK_OBJECT (uiinfo->widget),
+				    signal_name, uiinfo->moreinfo, uibdata->data ?
 				    uibdata->data : uiinfo->user_data);
 }
 
@@ -2361,7 +2425,7 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 	 * We fall back to icon view if a certain listing type is not supported.
 	 * Be sure to keep this in sync with the uiinfo arrays in glayout.c.
 	 */
-	
+
 	if (panel->list_type == list_brief)
 		gtk_toggle_button_set_active (
 			GTK_TOGGLE_BUTTON (panel_view_toolbar_uiinfo[1].widget), TRUE);
@@ -2523,7 +2587,7 @@ void
 x_reset_sort_labels (WPanel *panel)
 {
 	int page;
-		
+
 	if (!panel->notebook)
 		return;
 
