@@ -144,7 +144,7 @@
 #include "gsession.h"
 #endif
 
-/* "$Id: main.c,v 1.97 1999/06/22 21:02:39 unammx Exp $" */
+/* "$Id: main.c,v 1.99 1999/08/31 22:55:19 unammx Exp $" */
 
 /* When the modes are active, left_panel, right_panel and tree_panel */
 /* Point to a proper data structure.  You should check with the functions */
@@ -174,7 +174,11 @@ volatile int quit = 0;
 int show_all_if_ambiguous = 0;
 
 /* Set when cd symlink following is desirable (bash mode) */
+#ifndef HAVE_GNOME
 int cd_symlinks = 1;
+#else
+int cd_symlinks = 0;
+#endif
 
 /* If set then dialogs just clean the screen when refreshing, else */
 /* they do a complete refresh, refreshing all the parts of the program */
@@ -2518,6 +2522,9 @@ print_mc_usage (void)
 #ifdef HAVE_SUBSHELL_SUPPORT
     "-X, --dbgsubshell  [DEVEL-ONLY: Debug the subshell].\n"
 #endif
+    "\n"
+    "Please send any bug reports (including the output of `mc -V')\n"
+    "to mc-bugs@nuclecu.unam.mx\n"
     );
 }
 
@@ -3156,7 +3163,7 @@ main (int argc, char *argv [])
 #   ifndef HAVE_X
     if (console_flag)
 	handle_console (CONSOLE_DONE);
-    putchar ('\r');  /* Hack to make shell's prompt start at left of screen */
+    putchar ('\n');  /* Hack to make shell's prompt start at left of screen */
 #   endif
 
 #ifdef _OS_NT
