@@ -69,7 +69,7 @@ extern int tree_panel_visible;
 extern int we_can_afford_the_speed;
 #endif
 
-/* "$Id: setup.c,v 1.27 1999/04/06 01:00:34 unammx Exp $" */
+/* "$Id: setup.c,v 1.30 1999/08/31 22:55:19 unammx Exp $" */
 
 #ifdef USE_VFS
 extern int vfs_timeout;
@@ -204,7 +204,9 @@ static struct {
     { "drop_menus", &drop_menus },
     { "wrap_mode",  &global_wrap_mode},
     { "old_esc_mode", &old_esc_mode },
+#ifndef HAVE_GNOME
     { "cd_symlinks", &cd_symlinks },
+#endif
     { "show_all_if_ambiguous", &show_all_if_ambiguous },
     { "have_fast_cpu", &have_fast_cpu },
     { "iconify_on_exec", &iconify_on_exec },
@@ -254,8 +256,12 @@ static struct {
 
 #ifdef HAVE_GNOME
     { "desktop_use_shaped_icons", &desktop_use_shaped_icons },
+    { "desktop_use_shaped_text", &desktop_use_shaped_text },
     { "desktop_auto_placement", &desktop_auto_placement },
     { "desktop_snap_icons", &desktop_snap_icons },
+    { "desktop_arr_r2l", &desktop_arr_r2l },
+    { "desktop_arr_b2t", &desktop_arr_b2t },
+    { "desktop_arr_rows", &desktop_arr_rows },
     { "tree_panel_visible", &tree_panel_visible },
     { "we_can_afford_the_speed", &we_can_afford_the_speed },
 #else
