@@ -59,7 +59,7 @@ int tree_navigation_flag;
 static int tree_callback (Dlg_head *h, WTree *tree, int msg, int par);
 #define tcallback (callback_fn) tree_callback
 
-/* "$Id: tree.c,v 1.20 1999/04/10 00:54:28 unammx Exp $" */
+/* "$Id: tree.c,v 1.21 1999/04/16 22:02:21 unammx Exp $" */
 
 static tree_entry *back_ptr (tree_entry *ptr, int *count)
 {
@@ -176,15 +176,9 @@ void tree_destroy (WTree *tree)
 /* Loads the .mc.tree file */
 void load_tree (WTree *tree)
 {
-    char *filename;
-    int v;
-    
-    filename = concat_dir_and_file (home_dir, MC_TREE);
-    v = tree_store_load (filename);
-    g_free (filename);
+    tree_store_load ();
 
     tree->selected_ptr = tree->store->tree_first;
-    
     tree_chdir (tree, home_dir);
 }
 
@@ -192,11 +186,8 @@ void load_tree (WTree *tree)
 void save_tree (WTree *tree)
 {
     int error;
-    char *filename;
     
-    filename = concat_dir_and_file (home_dir, MC_TREE);
-    error = tree_store_save (filename);
-   g_free (filename);
+    error = tree_store_save ();
 
     if (error){
 	fprintf (stderr, _("Can't open the %s file for writing:\n%s\n"), MC_TREE,
