@@ -76,7 +76,7 @@
 #include "user.h"		/* expand_format */
 #include "../vfs/vfs.h"
 
-/* "$Id: util.c,v 1.27 1999/06/01 22:39:14 unammx Exp $" */
+/* "$Id: util.c,v 1.28 1999/08/06 19:24:05 norbert Exp $" */
 
 char app_text [] = "Midnight-Commander";
 
@@ -350,7 +350,8 @@ strip_password (char *p, int has_prefix)
         size_t len;
     } prefixes[] = { {"/#ftp:", 6},
 		     {"/#mc:", 5},
-		     {"ftp://", 6}
+		     {"ftp://", 6},
+		     {"/#smb:", 6},
     };
     char *at, *inner_colon, *dir;
     int i;
@@ -591,10 +592,6 @@ char *load_file (char *filename)
 short-month-name sizes for different locales */ 
 size_t i18n_checktimelength (void)
 {
-#define MAX_I18NTIMELENGTH 14
-#define MIN_I18NTIMELENGTH 10
-#define STD_I18NTIMELENGTH 12
-
     size_t length, a, b;
     char buf [MAX_I18NTIMELENGTH + 1];
     time_t testtime = time (NULL);
