@@ -23,7 +23,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
  */
-/* "$Id: widget.c,v 1.31 1999/05/03 18:57:57 unammx Exp $" */
+/* "$Id: widget.c,v 1.33 1999/08/06 19:24:05 norbert Exp $" */
 
 #include <config.h>
 #include <string.h>
@@ -990,7 +990,15 @@ void history_put (char *input_name, Hist *h)
 
 /* {{{ history display */
 
-static const char history_title[] = " History ";
+static char *
+i18n_htitle (void)
+{
+    static char *history_title = NULL;
+
+    if (history_title == NULL)
+	history_title = _(" History ");
+    return history_title;	
+}
 
 static int
 history_callback (Dlg_head * h, int Par, int Msg)
@@ -1002,8 +1010,8 @@ history_callback (Dlg_head * h, int Par, int Msg)
 	dlg_erase (h);
 	draw_box (h, 0, 0, h->lines, h->cols);
 	attrset (COLOR_HOT_NORMAL);
-	dlg_move (h, 0, (h->cols - strlen (history_title)) / 2);
-	printw ((char *) history_title);
+	dlg_move (h, 0, (h->cols - strlen (i18n_htitle())) / 2);
+	printw (i18n_htitle());
 	break;
     }
 #endif
@@ -1015,7 +1023,7 @@ static inline int listbox_fwd (WListbox *l);
 char *show_hist (Hist *history, int widget_x, int widget_y)
 {
     Hist *hi, *z;
-    int maxlen = strlen(history_title), i, count = 0;
+    size_t maxlen = strlen (i18n_htitle()), i, count = 0;
     int x, y, w, h;
     char *q, *r = 0;
     Dlg_head *query_dlg;
@@ -1147,11 +1155,28 @@ input_enable_update (WInput *in)
     update_input (in, 0);
 }
 
+#define ELEMENTS(a)    (sizeof(a)/sizeof(a[0]))
+
 int
 push_history (WInput *in, char *text)
 {
+    static int i18n;
+    /* input widget where urls with passwords are entered without any
+       vfs prefix */
+    static const char *password_input_fields[] = {
+	" Link to a remote machine ",
+	" FTP to machine ",
+	" SMB link to machine "
+    };
     Hist *new;
     char *p;
+    int i;
+
+    if (!i18n) {
+	i18n = 1;
+        for (i = 0; i < ELEMENTS(password_input_fields); i++)
+	    password_input_fields[i] = _(password_input_fields[i]);
+    }
     
     for (p = text; *p == ' ' || *p == '\t'; p++);
     if (!*p)
@@ -1169,22 +1194,23 @@ push_history (WInput *in, char *text)
     new->next = 0;
     new->prev = in->history;
     new->text = g_strdup (text);
-    if (in->history_name){
-#ifdef HAVE_GNOME
-	    if (strcmp (in->history_name + 4, _(" Link to a remote machine ")) == 0 ||
-		strcmp (in->history_name + 4, _(" FTP to machine ")) == 0)
-#else
-		    if (strcmp (in->history_name + 3, _(" Link to a remote machine ")) == 0 ||
-			strcmp (in->history_name + 3, _(" FTP to machine ")) == 0)
-#endif
-			    strip_password (new->text, 0);
-		    else
-			    strip_password (new->text, 1);
+    if (in->history_name) {
+        p = in->history_name + 3;
+        for (i = 0; i < ELEMENTS(password_input_fields); i++)
+            if (strcmp (p, password_input_fields[i]) == 0)
+                break;
+        if (i < ELEMENTS(password_input_fields))
+            strip_password (new->text, 0);
+        else
+            strip_password (new->text, 1);
     }
+    
     in->history = new;
     return 2;
 }
 
+#undef ELEMENTS
+
 /* Cleans the input line and adds the current text to the history */
 void
 new_input (WInput *in)
