@@ -172,36 +172,38 @@ static char *
 translate_path (struct connection *bucket, char *remote_path)
 {
     char *p;
-    static char buf[255]; /* No one ever needs more ;-).
-                             Actually I consider this static a bug 
-                             -- Norbert */
     
-    if (!bucket->remote_is_amiga || strlen (remote_path) >= sizeof (buf) - 1)
-	return remote_path;
+    if (!bucket->remote_is_amiga)
+	return g_strdup (remote_path);
     else {
+	char *ret;
+
 	if (logfile) {
 	    fprintf (logfile, "MC -- translate_path: %s\n", remote_path);
 	    fflush (logfile);
 	}
 
+	/*
+	 * Don't change "/" into "", e.g. "CWD " would be
+	 * invalid.
+	 */
         if (*remote_path == '/' && remote_path[1] == '\0')
-	    return "."; /* Don't change "/" into "", e.g. "CWD " would be
-                           invalid. */
+	    return g_strdup ("."); 
 
 	/* strip leading slash */
 	if (*remote_path == '/')
-	    strcpy (buf, remote_path + 1);
+	    ret = g_strdup (remote_path + 1);
 	else
-	    strcpy (buf, remote_path);
+	    ret = g_strdup (remote_path);
 
 	/* replace first occurance of ":/" with ":" */
-	if ((p = strchr (buf, ':')) && *(p + 1) == '/')
+	if ((p = strchr (ret, ':')) && *(p + 1) == '/')
 	    strcpy (p + 1, p + 2);
 
 	/* strip trailing "/." */
-	if ((p = strrchr (buf, '/')) && *(p + 1) == '.' && *(p + 2) == '\0')
+	if ((p = strrchr (ret, '/')) && *(p + 1) == '.' && *(p + 2) == '\0')
 	    *p = '\0';
-	return buf;
+	return ret;
     }
 }
 
@@ -1033,10 +1035,12 @@ open_data_connection (struct connection *bucket, char *cmd, char *remote,
 	if (j != CONTINUE)
 	    return -1;
     }
-    if (remote)
-        j = command (bucket, WAIT_REPLY, "%s %s", cmd, 
-		translate_path (bucket, remote));
-    else
+    if (remote){
+	char *path = translate_path (bucket, remote);
+	
+        j = command (bucket, WAIT_REPLY, "%s %s", cmd, path);
+	g_free (path);
+    } else
     	j = command (bucket, WAIT_REPLY, "%s", cmd);
     if (j != PRELIM)
 	    ERRNOR (EPERM, -1);
@@ -1089,12 +1093,13 @@ resolve_symlink_without_ls_options(struct connection *bucket, struct dir *dir)
     struct linklist *flist;
     struct direntry *fe, *fel;
     char tmp[MC_MAXPATHLEN];
+    int depth;
     
     dir->symlink_status = FTPFS_RESOLVING_SYMLINKS;
     for (flist = dir->file_list->next; flist != dir->file_list; flist = flist->next) {
         /* flist->data->l_stat is alread initialized with 0 */
         fel = flist->data;
-        if (S_ISLNK(fel->s.st_mode)) {
+        if (S_ISLNK(fel->s.st_mode) && fel->linkname) {
   	    if (fel->linkname[0] == '/') {
 		if (strlen (fel->linkname) >= MC_MAXPATHLEN)
 		    continue;
@@ -1107,7 +1112,7 @@ resolve_symlink_without_ls_options(struct connection *bucket, struct dir *dir)
                    strcat (tmp, "/");
                 strcat (tmp + 1, fel->linkname);
 	    }
-	    for ( ;; ) {
+	    for ( depth = 0; depth < 100; depth++) { /* depth protects against recursive symbolic links */
 		canonicalize_pathname (tmp);
                 fe = _get_file_entry(bucket, tmp, 0, 0);
                 if (fe) {
@@ -1691,6 +1696,7 @@ send_ftp_command(char *filename, char *cmd, int flags)
 	return -1;
     p = translate_path (bucket, remote_path);
     r = command (bucket, WAIT_REPLY, cmd, p);
+    g_free (p);
     g_free(remote_path);
     vfs_add_noncurrent_stamps (&vfs_ftpfs_ops, (vfsid) bucket, NULL);
     if (flags & OPT_IGNORE_ERROR)
@@ -1782,7 +1788,8 @@ ftpfs_chdir_internal (struct connection *bucket ,char *remote_path)
 
     p = translate_path (bucket, remote_path);
     r = command (bucket, WAIT_REPLY, "CWD %s", p);
-
+    g_free (p);
+    
     if (r != COMPLETE) {
 	my_errno = EIO;
     } else {
